from __future__ import print_function
from future.builtins import str
from future import standard_library
standard_library.install_hooks()
from future.builtins import object
import os, sys
import re
import logging
import pickle
import inspect

log = logging

def safe_import(path):
    module = path.split('.')
    g = __import__(module[0], fromlist=['*'])
    s = [module[0]]
    for i in module[1:]:
        mod = g
        if hasattr(mod, i):
            g = getattr(mod, i)
        else:
            s.append(i)
            g = __import__('.'.join(s), fromlist=['*'])
    return mod, g
        
def import_mod_attr(path):
    """
    Import string format module, e.g. 'uliweb.orm' or an object
    return module object and object
    """
    if isinstance(path, str):
        module, func = path.rsplit('.', 1)
        mod = __import__(module, fromlist=['*'])
        f = getattr(mod, func)
    else:
        f = path
        mod = inspect.getmodule(path)
    return mod, f

def import_attr(func):
    mod, f = import_mod_attr(func)
    return f

def myimport(module):
    mod = __import__(module, fromlist=['*'])
    return mod

def install(packages):
    from pkg_resources import load_entry_point
    
    load = load_entry_point('setuptools', 'console_scripts', 'easy_install')
    load(packages)

class MyPkg(object):
    @staticmethod
    def resource_filename(module, path):
        mod = myimport(module)
        p = os.path.dirname(mod.__file__)
        if path:
            return os.path.join(p, path)
        else:
            return p
    
    @staticmethod
    def resource_listdir(module, path):
        d = MyPkg.resource_filename(module, path)
        return os.listdir(d)
    
    @staticmethod
    def resource_isdir(module, path):
        d = MyPkg.resource_filename(module, path)
        return os.path.isdir(d)

try:
    import pkg_resources as pkg
except:
    pkg = MyPkg

def extract_file(module, path, dist, verbose=False, replace=True):
    outf = os.path.join(dist, os.path.basename(path))
#    d = pkg.get_distribution(module)
#    if d.has_metadata('zip-safe'):
#        f = open(outf, 'wb')
#        f.write(pkg.resource_string(module, path))
#        f.close()
#        if verbose:
#            print 'Info : Extract %s/%s to %s' % (module, path, outf)
#    else:
    import shutil

    inf = pkg.resource_filename(module, path)
    sfile = os.path.basename(inf)
    if os.path.isdir(dist):
        dfile = os.path.join(dist, sfile)
    else:
        dfile = dist
    f = os.path.exists(dfile)
    if replace or not f:
        shutil.copy2(inf, dfile)
        if verbose:
            print('Copy %s to %s' % (inf, dfile))
  
def extract_dirs(mod, path, dst, verbose=False, exclude=None, exclude_ext=None, recursion=True, replace=True):
    """
    mod name
    path mod path
    dst output directory
    resursion True will extract all sub module of mod
    """
    default_exclude = ['.svn', '_svn', '.git']
    default_exclude_ext = ['.pyc', '.pyo', '.bak', '.tmp']
    exclude = exclude or []
    exclude_ext = exclude_ext or []
#    log = logging.getLogger('uliweb')
    if not os.path.exists(dst):
        os.makedirs(dst)
        if verbose:
            print('Make directory %s' % dst)
    for r in pkg.resource_listdir(mod, path):
        if r in exclude or r in default_exclude:
            continue
        fpath = os.path.join(path, r)
        if pkg.resource_isdir(mod, fpath):
            if recursion:
                extract_dirs(mod, fpath, os.path.join(dst, r), verbose, exclude, exclude_ext, recursion, replace)
        else:
            ext = os.path.splitext(fpath)[1]
            if ext in exclude_ext or ext in default_exclude_ext:
                continue
            extract_file(mod, fpath, dst, verbose, replace)

def match(f, patterns):
    from fnmatch import fnmatch
    
    flag = False
    for x in patterns:
        if fnmatch(f, x):
            return True
        
def walk_dirs(path, include=None, include_ext=None, exclude=None, 
        exclude_ext=None, recursion=True, file_only=False):
    """
    path directory path
    resursion True will extract all sub module of mod
    """
    default_exclude = ['.svn', '_svn', '.git']
    default_exclude_ext = ['.pyc', '.pyo', '.bak', '.tmp']
    exclude = exclude or []
    exclude_ext = exclude_ext or []
    include_ext = include_ext or []
    include = include or []

    if not os.path.exists(path):
        raise StopIteration
    
    for r in os.listdir(path):
        if match(r, exclude) or r in default_exclude:
            continue
        if include and r not in include:
            continue
        fpath = os.path.join(path, r)
        if os.path.isdir(fpath):
            if not file_only:
                yield os.path.normpath(fpath).replace('\\', '/')
            if recursion:
                for f in walk_dirs(fpath, include, include_ext, exclude, 
                    exclude_ext, recursion, file_only):
                    yield os.path.normpath(f).replace('\\', '/')
        else:
            ext = os.path.splitext(fpath)[1]
            if ext in exclude_ext or ext in default_exclude_ext:
                continue
            if include_ext and ext not in include_ext:
                continue
            yield os.path.normpath(fpath).replace('\\', '/')

def copy_dir(src, dst, verbose=False, check=False, processor=None):
    import shutil

#    log = logging.getLogger('uliweb')

    def _md5(filename):
        try:
            import hashlib
            a = hashlib.md5()
        except ImportError:
            import md5
            a = md5.new()
            
        a.update(file(filename, 'rb').read())
        return a.digest()
    
    if not os.path.exists(dst):
        os.makedirs(dst)

    if verbose:
        print("Processing %s" % src)
        
    for r in os.listdir(src):
        if r in ['.svn', '_svn', '.git']:
            continue
        fpath = os.path.join(src, r)
        
        if os.path.isdir(fpath):
            if os.path.abspath(fpath) != os.path.abspath(dst):
                copy_dir(fpath, os.path.join(dst, r), verbose, check, processor)
            else:
                continue
        else:
            ext = os.path.splitext(fpath)[1]
            if ext in ['.pyc', '.pyo', '.bak', '.tmp']:
                continue
            df = os.path.join(dst, r)
            if check:
                if os.path.exists(df):
                    a = _md5(fpath)
                    b = _md5(df)
                    if a != b:
                        print("Error: Target file %s is already existed, and "
                            "it not same as source one %s, so copy failed" % (fpath, dst))
                else:
                    if processor:
                        if processor(fpath, dst, df):
                            continue
                    shutil.copy2(fpath, dst)
                    if verbose:
                        print("Copy %s to %s" % (fpath, dst))
                    
            else:
                if processor:
                    if processor(fpath, dst, df):
                        continue
                shutil.copy2(fpath, dst)
                if verbose:
                    print("Copy %s to %s" % (fpath, dst))

def copy_dir_with_check(dirs, dst, verbose=False, check=True, processor=None):
#    log = logging.getLogger('uliweb')

    for d in dirs:
        if not os.path.exists(d):
            continue

        copy_dir(d, dst, verbose, check, processor)

def is_pyfile_exist(dir, pymodule):
    path = os.path.join(dir, '%s.py' % pymodule)
    if not os.path.exists(path):
        path = os.path.join(dir, '%s.pyc' % pymodule)
        if not os.path.exists(path):
            path = os.path.join(dir, '%s.pyo' % pymodule)
            if not os.path.exists(path):
                return False
    return True
    
def wraps(src):
    def _f(des):
        def f(*args, **kwargs):
            from uliweb import application
            if application:
                env = application.get_view_env()
                for k, v in env.items():
                    src.__globals__[k] = v
                
                src.__globals__['env'] = env
            return des(*args, **kwargs)
        
        f.__name__ = src.__name__
        f.__globals__.update(src.__globals__)
        f.__doc__ = src.__doc__
        f.__module__ = src.__module__
        f.__dict__.update(src.__dict__)
        return f
    
    return _f

def timeit(func):
    log = logging.getLogger('uliweb.app')
    import time
    @wraps(func)
    def f(*args, **kwargs):
        begin = time.time()
        ret = func(*args, **kwargs)
        end = time.time()
        print(("%s.%s [%s]s" % (func.__module__, func.__name__, end-begin)))
        return ret
    return f

def get_var(key):
    def f():
        from uliweb import settings
        
        return settings.get_var(key)
    return f

def get_choice(choices, value, default=None):
    if callable(choices):
        choices = choices()
    return dict(choices).get(value, default)

def simple_value(v, encoding='utf-8', none=False):
    import datetime
    import decimal
    
    if callable(v):
        v = v()
    if isinstance(v, datetime.datetime):
        return v.strftime('%Y-%m-%d %H:%M:%S')
    elif isinstance(v, datetime.date):
        return v.strftime('%Y-%m-%d')
    elif isinstance(v, datetime.time):
        return v.strftime('%H:%M:%S')
    elif isinstance(v, decimal.Decimal):
        return str(v)
    elif isinstance(v, str):
        return v.encode(encoding)
    elif isinstance(v, (tuple, list)):
        s = []
        for x in v:
            s.append(simple_value(x, encoding, none))
        return s
    elif isinstance(v, dict):
        d = {}
        for k, v in v.items():
            d[simple_value(k)] = simple_value(v, encoding, none)
        return d
    elif v is None:
        if none:
            return v
        else:
            return ''
    else:
        return v
    
def str_value(v, encoding='utf-8', bool_int=True, none='NULL'):
    import datetime
    import decimal
    
    if callable(v):
        v = v()
    if isinstance(v, datetime.datetime):
        return v.strftime('%Y-%m-%d %H:%M:%S')
    elif isinstance(v, datetime.date):
        return v.strftime('%Y-%m-%d')
    elif isinstance(v, datetime.time):
        return v.strftime('%H:%M:%S')
    elif isinstance(v, decimal.Decimal):
        return str(v)
    elif isinstance(v, str):
        return v.encode(encoding)
    elif v is None:
        return none
    elif isinstance(v, bool):
        if bool_int:
            if v:
                return '1'
            else:
                return '0'
        else:
            return str(v)
    else:
        return str(v)

__caches__ = {}
def cache_get(key, func, _type='default'):
    global __caches__
    v = __caches__.setdefault(_type, {})
    if key in v and v[key]:
        return v[key]
    else:
        v[key] = func(key)
        return v[key]
    
def norm_path(path):
    return os.path.normcase(os.path.abspath(path))

r_expand_path = re.compile('\$\{(\w+)\}')
def expand_path(path):
    """
    Auto search some variables defined in path string, such as:
        ${PROJECT}/files
        ${app_name}/files
    for ${PROJECT} will be replaced with uliweb application apps_dir directory
    and others will be treated as a normal python package, so uliweb will
    use pkg_resources to get the path of the package
    """
    from uliweb import application
    
    def replace(m):
        txt = m.groups()[0]
        if txt == 'PROJECT':
            return application.apps_dir
        else:
            return pkg.resource_filename(txt, '')
    return re.sub(r_expand_path, replace, path)

def date_in(d, dates):
    """
    compare if d in dates. dates should be a tuple or a list, for example:
        date_in(d, [d1, d2])
    and this function will execute:
        d1 <= d <= d2
    and if d is None, then return False
    """
    if not d:
        return False
    return dates[0] <= d <= dates[1]

class Serial(object):
    @classmethod
    def load(self, s):
        return pickle.loads(s)
    
    @classmethod
    def dump(self, v):
        return pickle.dumps(v, pickle.HIGHEST_PROTOCOL)

import future.moves.urllib.parse as urllib_parse
class QueryString(object):
    def __init__(self, url):
        self.url = url
        self.scheme, self.netloc, self.script_root, qs, self.anchor = self.parse()
        self.qs = urllib_parse.parse_qs(qs, True)
        
    def parse(self):
        return urllib_parse.urlsplit(self.url)
    
    def __getitem__(self, name):
        return self.qs.get(name, [])
    
    def __setitem__(self, name, value):
        self.qs[name] = [value]
    
    def set(self, name, value, replace=False):
        v = self.qs.setdefault(name, [])
        if replace:
            self.qs[name] = [value]
        else:
            v.append(value)
        return self

    def __str__(self):
        import urllib.request, urllib.parse, urllib.error
        
        qs = urllib.parse.urlencode(self.qs, True)
        return urllib_parse.urlunsplit((self.scheme, self.netloc, self.script_root, qs, self.anchor))
    
def query_string(url, replace=True, **kwargs):
    q = QueryString(url)
    for k, v in kwargs.items():
        q.set(k, v, replace)
    return str(q)

def camel_to_(s):
    """
    Convert CamelCase to camel_case
    """
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', s)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()
    
def get_uuid(type=4):
    """
    Get uuid value
    """
    import uuid
    
    name = 'uuid'+str(type)
    u = getattr(uuid, name)
    return u().hex

def pretty_dict(d, leading=' ', newline='\n', indent=0, tabstop=4, process=None):
    """
    Output pretty formatted dict, for example:
        
        d = {"a":"b",
            "c":{
                "d":"e",
                "f":"g",
                }
            }
        
    will output:
        
        a : 'b'
        c : 
            d : 'e'
            f : 'g'
        
    """
    for k, v in d.items():
        if process:
            k, v = process(k, v)
        if isinstance(v, dict):
            yield '%s%s : %s' % (indent*tabstop*leading, k, newline)
            for x in pretty_dict(v, leading=leading, newline=newline, indent=indent+1, tabstop=tabstop):
                yield x
            continue
        yield '%s%s : %s%s' % (indent*tabstop*leading, k, simple_value(v), newline)


#if __name__ == '__main__':
#    log.info('Info: info')
#    try:
#        1/0
#    except:
#        log.exception('1/0')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

"""Simple Markdown writer for reStructuredText."""

__docformat__ = 'reStructuredText'

from docutils import frontend, nodes, writers, languages
from docutils.parsers.rst import Directive, directives

class Writer(writers.Writer):

    supported = ('markdown',)
    """Formats this writer supports."""

    output = None
    """Final translated form of `document`."""

    # Add configuration settings for additional Markdown flavours here.
    settings_spec = (
        'Markdown-Specific Options',
        None,
        (('Extended Markdown syntax.',
          ['--extended-markdown'],
          {'default': 0, 'action': 'store_true',
           'validator': frontend.validate_boolean}),
         ('Strict Markdown syntax. Default: true',
          ['--strict-markdown'],
          {'default': 1, 'action': 'store_true',
           'validator': frontend.validate_boolean}),))

    def __init__(self):
        writers.Writer.__init__(self)
        self.translator_class = Translator

    def translate(self):
        visitor = self.translator_class(self.document)
        self.document.walkabout(visitor)
        self.output = visitor.astext()

#register toctree, in order to process sphinx

class toctree(nodes.General, nodes.Text):pass

class Toctree(Directive):
    has_content = True
    required_arguments = 0
    optional_arguments = 0
    final_argument_whitespace = False
    option_spec = {
        'maxdepth': int,
    }
    def run(self):
        max_depth = self.options.get('maxdepth', 1)
        text = ['{%% toc max_depth=%d %%}' % max_depth]+[x for x in self.content]+['{% endtoc %}']
        node = toctree('\n'.join(text))
        return [node]

directives.register_directive('toctree', Toctree)

class Translator(nodes.NodeVisitor):

    def __init__(self, document):
        nodes.NodeVisitor.__init__(self, document)
        self.settings = settings = document.settings
        lcode = settings.language_code
        self.language = languages.get_language(lcode, document.reporter)

        self.head = []
        self.body = []
        self.foot = []
        self.list_type = []
        self.indent = 0
        self.first_indent = True

        self.section_level = 0

        ##TODO docinfo items can go in a footer HTML element (store in self.foot).
        self._docinfo = {
            'title' : '',
            'subtitle' : '',
            'author' : [],
            'date' : '',
            'copyright' : '',
            'version' : '',
            }

        # Customise Markdown syntax here. Still need to add literal, term,
        # indent, problematic etc...
        self.defs = {
            'emphasis': ('*', '*'),   # Could also use ('_', '_')
            'problematic' : ('\n\n', '\n\n'),
            'strong' : ('**', '**'),  # Could also use ('__', '__')
            'subscript' : ('<sub>', '</sub>'),
            'superscript' : ('<sup>', '</sup>'),
            }

    # Utility methods

    def astext(self):
        """Return the final formatted document as a string."""
        return ''.join(self.head + self.body + self.foot)

    def deunicode(self, text):
        text = text.replace(u'\xa0', ' ')
        text = text.replace(u'\u2020', '\\(dg')
        return text

    def ensure_eol(self):
        """Ensure the last line in body is terminated by new line."""
        if self.body and self.body[-1][-1] != '\n':
            self.body.append('\n')

    def indent_text(self, text, indent=0, prefix='', first=True):
        """
        Splite text into lines, and indent them with prefix string
        """
        lines = text.splitlines()
        if first:
            return '\n'.join([' '*indent*4+prefix+x for x in lines])
        else:
            return '\n'.join([lines[0]] + [' '*indent*4+prefix+x for x in lines[1:]])
        
    def set_class_on_child(self, node, class_, index=0):
        """
        Set class `class_` on the visible child no. index of `node`.
        Do nothing if node has fewer children than `index`.
        """
        children = [n for n in node if not isinstance(n, nodes.Invisible)]
        try:
            child = children[index]
        except IndexError:
            return
        child['classes'].append(class_)

    def set_first_last(self, node):
        self.set_class_on_child(node, 'first', 0)
        self.set_class_on_child(node, 'last', -1)

    # Node visitor methods

    def visit_Text(self, node):
        text = node.astext()
        self.body.append(text)

    def depart_Text(self, node):
        pass

    def visit_comment(self, node):
        self.body.append('<!-- ' + node.astext() + ' -->\n')
        raise nodes.SkipNode

    def visit_docinfo_item(self, node, name):
        if name == 'author':
            self._docinfo[name].append(node.astext())
        else:
            self._docinfo[name] = node.astext()
        raise nodes.SkipNode

    def visit_document(self, node):
        pass

    def depart_document(self, node):
        pass

    def visit_emphasis(self, node):
        self.body.append(self.defs['emphasis'][0])

    def depart_emphasis(self, node):
        self.body.append(self.defs['emphasis'][1])

    def walk_text(self, node):
        backup = self.body
        self.body = []
        for c in node.children:
            c.walkabout(self)
        text = ''.join(self.body)
        self.body = backup
        return text
    
    def visit_paragraph(self, node):
        text = self.walk_text(node)
            
        #self.ensure_eol()
        if self.indent:
            self.body.append(self.indent_text(text, self.indent, first=self.first_indent))
            self.body.append('\n')
            self.first_indent = True
            raise nodes.SkipNode

    def depart_paragraph(self, node):
        self.body.append('\n\n')

    def visit_problematic(self, node):
        self.body.append(self.defs['problematic'][0])

    def depart_problematic(self, node):
        self.body.append(self.defs['problematic'][1])

    def visit_section(self, node):
        self.section_level += 1

    def depart_section(self, node):
        self.section_level -= 1

    def visit_strong(self, node):
        self.body.append(self.defs['strong'][0])

    def depart_strong(self, node):
        self.body.append(self.defs['strong'][1])

    def visit_subscript(self, node):
        self.body.append(self.defs['subscript'][0])

    def depart_subscript(self, node):
        self.body.append(self.defs['subscript'][1])

    def visit_subtitle(self, node):
        if isinstance(node.parent, nodes.document):
            self.visit_docinfo_item(node, 'subtitle')
            raise nodes.SkipNode

    def visit_superscript(self, node):
        self.body.append(self.defs['superscript'][0])

    def depart_superscript(self, node):
        self.body.append(self.defs['superscript'][1])

    def visit_system_message(self, node):
        # TODO add report_level
        #if node['level'] < self.document.reporter['writer'].report_level:
        #    Level is too low to display:
        #    raise nodes.SkipNode
        attr = {}
        backref_text = ''
        if node.hasattr('id'):
            attr['name'] = node['id']
        if node.hasattr('line'):
            line = ', line %s' % node['line']
        else:
            line = ''
        self.body.append('"System Message: %s/%s (%s:%s)"\n'
            % (node['type'], node['level'], node['source'], line))

    def depart_system_message(self, node):
        pass

    def visit_title(self, node):
        if self.section_level == 0:
            self.head.append(u'# {0}\n'.format(node.astext()))
            self._docinfo['title'] = node.astext()
            self.body.append('\n')
            raise nodes.SkipNode
        else:
            self.body.append(u'\n{0} {1}\n'.format((self.section_level+1)*'#',
                self.deunicode(node.astext()).split(' ', 1)[-1].strip()))
            self.body.append('\n')
            raise nodes.SkipNode

    def depart_title(self, node):
        self.body.append('\n')

    def visit_transition(self, node):
        # Simply replace a transition by a horizontal rule.
        # Could use three or more '*', '_' or '-'.
        self.body.append('\n---\n\n')
        raise nodes.SkipNode

    #parm support
    def visit_definition(self, node):
        self.body.append(self.indent*4*' ')
        self.indent += 1

    def depart_definition(self, node):
        self.body.append('\n')
        self.indent -= 1

    def visit_definition_list(self, node):
        self.body.append('\n')

    def depart_definition_list(self, node):
        self.body.append('\n')

    def visit_definition_list_item(self, node):
        pass

    def depart_definition_list_item(self, node):
        pass

    def visit_term(self, node):
        self.body.append(self.indent*4*' ')

    def depart_term(self, node):
        self.body.append(' --\n')

    def visit_enumerated_list(self, node):
        self.list_type.append('number')
        self.body.append('\n')
        self.indent += 1

    def depart_enumerated_list(self, node):
        self.list_type.pop()
        self.body.append('\n')
        self.indent -= 1

    def visit_bullet_list(self, node):
        self.list_type.append('bullet')
        self.body.append('\n')
        self.indent += 1
        
    def depart_bullet_list(self, node):
        self.list_type.pop()
        self.body.append('\n')
        self.indent -= 1

    def visit_list_item(self, node):
        if self.list_type[-1] == 'number':
            self.body.append((self.indent-1)*4*' '+'1. ')
        else:
            self.body.append((self.indent-1)*4*' '+'* ')
        self.first_indent = False

    def depart_list_item(self, node):
        self.first_indent = True

    def visit_literal_block(self, node):
        self.body.append('\n')
        self.body.append(self.indent_text('```\n'+node.astext()+'\n```', self.indent))
        self.body.append('\n\n')
        raise nodes.SkipNode

    def visit_literal(self, node):
        self.body.append('`')

    def depart_literal(self, node):
        self.body.append('`')
        
    def visit_block_quote(self, node):
        children = [n for n in node if not isinstance(n, nodes.Invisible)]
        if children[0].tagname not in ('bullet_list', 'enumerated_list', 'definition_list'):
            self.body.append('\n')
            self.body.append(self.indent_text(node.astext(), self.indent, '> '))
            self.body.append('\n')
            raise nodes.SkipNode
        
    def depart_block_quote(self, node):
        pass
    
    def visit_toctree(self, node):
        self.body.append('\n')
        self.body.append(node.astext().replace('.rst', '.md'))
        self.body.append('\n\n')
        raise nodes.SkipNode
        
    def depart_toctree(self, node):
        pass
    
    def visit_reference(self, node):
        url = node['refuri']
        if url.startswith('mailto:'):
            self.body.append('%s' % url)
        else:
            self.body.append('[%s](%s)' % (node.astext(), url))
        raise nodes.SkipNode
    
    def visit_note(self, node):
        self.body.append('\n')
        self.body.append(self.indent*4*' ')
        self.body.append('{% alert class=info %}\n')
        
    def depart_note(self, node):
        self.body.append(self.indent*4*' ')
        self.body.append('{% endalert %}\n\n')

    def visit_attention(self, node):
        self.body.append('\n')
        self.body.append(self.indent*4*' ')
        self.body.append('{% alert class=warning %}\n')
        
    def depart_attention(self, node):
        self.body.append(self.indent*4*' ')
        self.body.append('{% endalert %}\n\n')

    def visit_image(self, node):
        url = node['uri']
        self.body.append('\n![image](%s)\n\n' % url)
        raise nodes.SkipNode
        
    def depart_image(self, node):
        pass
    
    def visit_table(self, node):
        self.ensure_eol()
        self.body.append('\n')
    
    def depart_table(self, node):
        self.body.append('\n')
    
    def visit_row(self, node):
        self.body.append(self.indent*4*' ')
        self.body.append('||')
    
    def depart_row(self, node):
        self.body.append('\n')
        
    def visit_entry(self, node):
        self.body.append(' ')
        self.body.append(self.walk_text(node).strip())
        self.body.append(' ||')
        raise nodes.SkipNode
    
    def depart_entry(self, node):
        pass
        
    def visit_tgroup(self, node):
        pass
    
    def depart_tgroup(self, node):
        pass
    
    def visit_tbody(self, node):
        pass
    
    def depart_tbody(self, node):
        pass

    def visit_thead(self, node):
        pass
    
    def depart_thead(self, node):
        pass
    
    def visit_colspec(self, node):
        pass
    
    def depart_colspec(self, node):
        pass
    
# The following code adds visit/depart methods for any reSturcturedText element
# which we have not explicitly implemented above.

# All reStructuredText elements:
rst_elements = ('abbreviation', 'acronym', 'address', 'admonition',
    'attention', 'attribution', 'author', 'authors', 'block_quote', 
    'bullet_list', 'caption', 'caution', 'citation', 'citation_reference', 
    'classifier', 'colspec', 'comment', 'compound', 'contact', 'container', 
    'copyright', 'danger', 'date', 'decoration', 'definition',
    'definition_list', 'definition_list_item', 'description', 'docinfo', 
    'doctest_block', 'document', 'emphasis', 'entry', 'enumerated_list', 
    'error', 'field', 'field_body', 'field_list', 'field_name', 'figure', 
    'footer', 'footnote', 'footnote_reference', 'generated', 'header', 
    'hint', 'image', 'important', 'inline', 'label', 'legend', 'line', 
    'line_block', 'list_item', 'literal', 'literal_block', 'math', 
    'math_block', 'note', 'option' ,'option_argument', 'option_group', 
    'option_list', 'option_list_item', 'option_string', 'organization', 
    'paragraph', 'pending', 'problematic', 'raw', 'reference', 'revision', 
    'row', 'rubric', 'section', 'sidebar', 'status', 'strong', 'subscript', 
    'substitution_definition', 'substitution_reference', 'subtitle', 
    'superscript', 'system_message', 'table', 'target', 'tbody,' 'term', 
    'tgroup', 'thead', 'tip', 'title', 'title_reference', 'topic', 
    'transition','version','warning',)

##TODO Eventually we should silently ignore unsupported reStructuredText 
##     constructs and document somewhere that they are not supported.
##     In the meantime raise a warning *once* for each unsupported element.
_warned = set()

def visit_unsupported(self, node):
    node_type = node.__class__.__name__
    if node_type not in _warned:
        self.document.reporter.warning('\nThe ' + node_type + \
            ' element is not supported. \nText:\n%s\n---\n' % node.astext()[:70])
        _warned.add(node_type)
    raise nodes.SkipNode

for element in rst_elements:
    if not hasattr(Translator, 'visit_' + element):
        setattr(Translator, 'visit_' + element , visit_unsupported)




from __future__ import print_function
from __future__ import unicode_literals
import sys
sys.path.insert(0, '..')
from par.md import parseHtml
from parm.md_ext import include

def test_include_1():
    """
    >>> text = '''
    ... {% include file=test.js, lines=1-2 5 10-, language=python, class=linenums  %}
    ... {% endinclude %}
    ... '''
    >>> blocks = {'include':include}
    >>> print (parseHtml(text, '%(body)s', block_callback=blocks))
    <BLANKLINE>
    <pre class="linenums"><code class="language-python">$(function(){
        $('.div').on('click', function(e){
    ...
                url:'/test',
    ...
            });
        });
    });</code></pre>
    <BLANKLINE>
    """

def test_include_2():
    """
    >>> text = '''
    ... {% include file=test.js %}
    ... {% endinclude %}
    ... '''
    >>> blocks = {'include':include}
    >>> print (parseHtml(text, '%(body)s', block_callback=blocks))
    <BLANKLINE>
    <pre><code>$(function(){
        $('.div').on('click', function(e){
            e.preventDefault();
            $.ajax({
                url:'/test',
                dataType:'json',
                success:function(result){
                    show_message(result);
                }
            });
        });
    });</code></pre>
    <BLANKLINE>
    """

def test_include_3():
    """
    >>> text = '''
    ... {% include file=test.js%}
    ... \$\('\.div'\)...^\}\);
    ... {% endinclude %}
    ... '''
    >>> blocks = {'include':include}
    >>> print (parseHtml(text, '%(body)s', block_callback=blocks))
    <BLANKLINE>
    <pre><code>    $('.div').on('click', function(e){
            e.preventDefault();
            $.ajax({
                url:'/test',
                dataType:'json',
                success:function(result){
                    show_message(result);
                }
            });
        });
    });</code></pre>
    <BLANKLINE>
    """
    
def test_include_4():
    """
    >>> text = '''
    ... {% include file=test.js%}
    ... ...e\.preventDefault
    ... {% endinclude %}
    ... '''
    >>> blocks = {'include':include}
    >>> print (parseHtml(text, '%(body)s', block_callback=blocks))
    <BLANKLINE>
    <pre><code>$(function(){
        $('.div').on('click', function(e){
            e.preventDefault();
    ...</code></pre>
    <BLANKLINE>
    """

def test_include_5():
    """
    >>> text = '''
    ... {% include file=test.js, lines=5-6%}
    ... ...e\.preventDefault
    ... {% endinclude %}
    ... '''
    >>> blocks = {'include':include}
    >>> print (parseHtml(text, '%(body)s', block_callback=blocks))
    <BLANKLINE>
    <pre><code>$(function(){
        $('.div').on('click', function(e){
            e.preventDefault();
    ...
                url:'/test',
                dataType:'json',
    ...</code></pre>
    <BLANKLINE>
    """

def test_include_6():
    """
    >>> text = '''
    ... {% include file=test.js %}
    ... \$\('\.div'\)...\$\.ajax
    ... {% endinclude %}
    ... '''
    >>> blocks = {'include':include}
    >>> print (parseHtml(text, '%(body)s', block_callback=blocks))
    <BLANKLINE>
    <pre><code>    $('.div').on('click', function(e){
            e.preventDefault();
            $.ajax({
    ...</code></pre>
    <BLANKLINE>
    """

def test_include_7():
    """
    >>> text = '''
    ... {% include file=test.js %}
    ... \$\('\.div'\)!...\$\.ajax!
    ... {% endinclude %}
    ... '''
    >>> blocks = {'include':include}
    >>> print (parseHtml(text, '%(body)s', block_callback=blocks))
    <BLANKLINE>
    <pre><code>...
            e.preventDefault();
    ...</code></pre>
    <BLANKLINE>
    """

def test_include_8():
    """
    >>> text = '''
    ... {% include file=test1.js %}
    ... <script>...</script>
    ... {% endinclude %}
    ... '''
    >>> blocks = {'include':include}
    >>> print (parseHtml(text, '%(body)s', block_callback=blocks))
    <BLANKLINE>
    <pre><code>...
    &lt;script&gt;
    avalon.define("blog", function(vm){
        vm.name = 'hello';
    });
    &lt;/script&gt;
    ...</code></pre>
    <BLANKLINE>
    """




from __future__ import print_function
from __future__ import absolute_import
from __future__ import unicode_literals
from future.builtins import input
from future.builtins import str
from future import standard_library
standard_library.install_hooks()
from future.builtins import object
##################################################################
# This module is desired by Django
##################################################################
import sys, os
from optparse import make_option, OptionParser, IndentedHelpFormatter
import logging
import inspect
from future.utils import with_metaclass

log = logging

def handle_default_options(options):
    """
    Include any default options that all commands should accept here
    so that ManagementUtility can handle them before searching for
    user commands.

    """
    pass

class CommandError(Exception):
    """
    Exception class indicating a problem while executing a management
    command.

    If this exception is raised during the execution of a management
    command, it will be caught and turned into a nicely-printed error
    message to the appropriate output stream (i.e., stderr); as a
    result, raising this exception (with a sensible description of the
    error) is the preferred way to indicate that something has gone
    wrong in the execution of a command.

    """
    pass

def get_answer(message, answers='Yn', default='Y', quit=''):
    """
    Get an answer from stdin, the answers should be 'Y/n' etc.
    If you don't want the user can quit in the loop, then quit should be None.
    """
    if quit and quit not in answers:
        answers = answers + quit
        
    message = message + '(' + '/'.join(answers) + ')[' + default + ']:'
    ans = input(message).strip().upper()
    if default and not ans:
        ans = default.upper()
    while ans not in answers.upper():
        ans = input(message).strip().upper()
    if quit and ans == quit.upper():
        print("Command be cancelled!")
        sys.exit(1)
    return ans

def get_input(prompt, default=None, choices=None, option_value=None):
    """
    If option_value is not None, then return it. Otherwise get the result from 
    input.
    """
    if option_value is not None:
        return option_value
    
    choices = choices or []
    while 1:
        r = input(prompt+' ').strip()
        if not r and default is not None:
            return default
        if choices:
            if r not in choices:
                r = None
            else:
                break
        else:
            break
    return r

__commands__ = {}

def get_commands(modules):
    global __commands__
    
    def check(c):
        return (inspect.isclass(c) and 
            issubclass(c, Command) and c is not Command and c is not CommandManager)
    
    def find_mod_commands(mod):
        for name in dir(mod):
            c = getattr(mod, name)
            if check(c):
                register_command(c)
        
    def collect_commands():
        for m in modules:
            try:
                mod = __import__(m, fromlist=['*'])
            except ImportError as e:
                if not str(e).startswith('No module named'):
                    import traceback
                    traceback.print_exc()
                continue
            
            find_mod_commands(mod)

    collect_commands()
    return __commands__
    
def register_command(kclass):
    global __commands__
    __commands__[kclass.name] = kclass

def call(prog_name, version, modules=None, args=None):
    from .commands import execute_command_line
    
    modules = modules or []
    
    if isinstance(args, str):
        import shlex
        args = shlex.split(args)
    
    execute_command_line(args or sys.argv, get_commands(modules), prog_name, version)

class CommandMetaclass(type):
    def __init__(cls, name, bases, dct):
        option_list = list(dct.get('option_list', []))
        for c in bases:
            if hasattr(c, 'option_list') and isinstance(c.option_list, list):
                option_list.extend(c.option_list)
        cls.option_list = option_list
        
class Command(with_metaclass(CommandMetaclass)):
    option_list = ()
    help = ''
    args = ''
    has_options = False

    def create_parser(self, prog_name, subcommand):
        """
        Create and return the ``OptionParser`` which will be used to
        parse the arguments to this command.
    
        """
        return OptionParser(prog=prog_name,
                            usage=self.usage(subcommand),
                            version='',
                            add_help_option = False,
                            option_list=self.option_list)
    def get_version(self):
        return "%s version is %s" % (self.prog_name, self.version)

    def usage(self, subcommand):
        """
        Return a brief description of how to use this command, by
        default from the attribute ``self.help``.
    
        """
        if self.has_options:
            usage = '%%prog %s [options] %s' % (subcommand, self.args)
        else:
            usage = '%%prog %s %s' % (subcommand, self.args)
        if self.help:
            return '%s\n\n%s' % (usage, self.help)
        else:
            return usage
    
    def print_help(self, prog_name, subcommand):
        """
        Print the help message for this command, derived from
        ``self.usage()``.
    
        """
        parser = self.create_parser(prog_name, subcommand)
        parser.print_help()
        
    def run_from_argv(self, prog, subcommand, global_options, argv):
        """
        Set up any environment changes requested, then run this command.
    
        """
        self.prog_name = prog
        parser = self.create_parser(prog, subcommand)
        options, args = parser.parse_args(argv)
        self.execute(args, options, global_options)
        
    def execute(self, args, options, global_options):
        try:
            self.handle(options, global_options, *args)
        except CommandError as e:
            log.exception(e)
            sys.exit(1)

    def handle(self, options, global_options, *args):
        """
        The actual logic of the command. Subclasses must implement
        this method.
    
        """
        raise NotImplementedError()
    
class NewFormatter(IndentedHelpFormatter):
    def format_heading(self, heading):
        return "%*s%s:\n" % (self.current_indent, "", 'Global Options')

class NewOptionParser(OptionParser):
    def _process_args(self, largs, rargs, values):
        while rargs:
            arg = rargs[0]
            longarg = False
            try:
                if arg[0:2] == "--" and len(arg) > 2:
                    # process a single long option (possibly with value(s))
                    # the superclass code pops the arg off rargs
                    longarg = True
                    self._process_long_opt(rargs, values)
                elif arg[:1] == "-" and len(arg) > 1:
                    # process a cluster of short options (possibly with
                    # value(s) for the last one only)
                    # the superclass code pops the arg off rargs
                    self._process_short_opts(rargs, values)
                else:
                    # it's either a non-default option or an arg
                    # either way, add it to the args list so we can keep
                    # dealing with options
                    del rargs[0]
                    raise Exception
            except:
                if longarg:
                    if '=' in arg:
                        del rargs[0]
                largs.append(arg)
                
class CommandManager(Command):
    usage_info = "%prog [global_options] [subcommand [options] [args]]"
    
    def __init__(self, argv=None, commands=None, prog_name=None, global_options=None, version=None):
        self.argv = argv
        self.version = version
        self.prog_name = prog_name or os.path.basename(self.argv[0])
        self.commands = commands
        self.global_options = global_options
    
    def get_commands(self):
        if callable(self.commands):
            commands = self.commands()
        else:
            commands = self.commands
        return commands
    
    def print_help_info(self, global_options):
        """
        Returns the script's main help text, as a string.
        """
        usage = ['',"Type '%s help <subcommand>' for help on a specific subcommand." % self.prog_name,'']
        usage.append('Available subcommands:')
        commands = list(self.get_commands().keys())
        commands.sort()
        for cmd in commands:
            usage.append('  %s' % cmd)
        return '\n'.join(usage)
    
    def fetch_command(self, global_options, subcommand):
        """
        Tries to fetch the given subcommand, printing a message with the
        appropriate command called from the command line (usually
        "uliweb") if it can't be found.
        """
        commands = self.get_commands()
        try:
            klass = commands[subcommand]
        except KeyError:
            sys.stderr.write("Unknown command: %r\nType '%s help' for usage.\n" % \
                (subcommand, self.prog_name))
            sys.exit(1)
        return klass
    
    def execute(self):
        """
        Given the command-line arguments, this figures out which subcommand is
        being run, creates a parser appropriate to that command, and runs it.
        """
        # Preprocess options to extract --settings and --pythonpath.
        # These options could affect the commands that are available, so they
        # must be processed early.
        parser = NewOptionParser(prog=self.prog_name,
                             usage=self.usage_info,
#                             version=self.get_version(),
                             formatter = NewFormatter(),
                             add_help_option = False,
                             option_list=self.option_list)
        
        if not self.global_options:
            global_options, args = parser.parse_args(self.argv)
            handle_default_options(global_options)
            args = args[1:]
        else:
            global_options = self.global_options
            args = self.argv
    
        def print_help(global_options):
            parser.print_help()
            sys.stderr.write(self.print_help_info(global_options) + '\n')
            sys.exit(1)
            
        if len(args) == 0:
            if global_options.version:
                print(self.get_version())
                sys.exit(1)
            else:
                print_help(global_options)
                sys.ext(1)
    
        try:
            subcommand = args[0]
        except IndexError:
            subcommand = 'help' # Display help if no arguments were given.
    
        if subcommand == 'help':
            if len(args) > 1:
                command = self.fetch_command(global_options, args[1])
                if issubclass(command, CommandManager):
                    cmd = command(['help'], None, '%s %s' % (self.prog_name, args[1]), global_options=global_options)
                    cmd.execute()
                else:
                    command().print_help(self.prog_name, args[1])
                sys.exit(1)
            else:
                print_help(global_options)
        if global_options.help:
            print_help(global_options)
        else:
            command = self.fetch_command(global_options, subcommand)
            if issubclass(command, CommandManager):
                cmd = command(args[1:], None, '%s %s' % (self.prog_name, subcommand), global_options=global_options)
                cmd.execute()
            else:
                cmd = command()
                cmd.run_from_argv(self.prog_name, subcommand, global_options, args[1:])
    
class ApplicationCommandManager(CommandManager):
    option_list = (
        make_option('--help', action='store_true', dest='help',
            help='show this help message and exit.'),
        make_option('-v', '--verbose', action='store_true', 
            help='Output the result in verbose mode.'),
        make_option('--version', action='store_true', dest='version',
            help="show program's version number and exit."),
    )
    help = ''
    args = ''
    
def execute_command_line(argv=None, commands=None, prog_name=None, version=None):
    m = ApplicationCommandManager(argv, commands, prog_name, version=version)
    m.execute()
    
if __name__ == '__main__':
    execute_command_line(sys.argv)


from __future__ import absolute_import
from __future__ import unicode_literals
from future.builtins import range, open
from .utils import log
import re
import os

def code_comment(visitor, items):
    """
    Format:
        
        [[code-comment(target=pre element id)]]:
            key : value
            key : value
            
        or:
            
        [[code-comment(pre element id)]]:
            key : value
            key : value
        
    """
    from uliweb import json_dumps
    
    txt = []
    data = {}
    txt.append('<script type="text/code-comment">')
    for x in items:
        d = {}
        for line in x['body'].splitlines():
            if line.strip():
                k, v = line.split(':', 1)
                k = k.strip()
                if '=' in v:
                    title, v = v.split('=', 1)
                    title = title.strip()
                else:
                    title = k
                v = visitor.parse_text(v.strip(), 'article')
                d[k] = {'title':title, 'content':v}
        if len(x['kwargs']) == 1 and list(x['kwargs'].keys())[0] != 'target':
            key = list(x['kwargs'].keys())[0]
        else:
            key = x['kwargs'].get('target', '')
        if key in data:
            data[key] = data[key].update(d)
        else:
            data[key] = d
    txt.append(json_dumps(data))
    txt.append('</script>')
    return '\n'.join(txt)

def new_code_comment(visitor, block):
    """
    Format:
        
        {% code-comment target=pre element id %}
        key : value
        key : value
        {% endcode-comment %}    
        
    """
    from uliweb import json_dumps
    
    if 'new' in block:
        
        txt = []
        data = {}
        txt.append('<script type="text/code-comment">')
        d = {}
        for line in block['body'].splitlines():
            if line.strip():
                k, v = line.split(':', 1)
                k = k.strip()
                if '=' in v:
                    title, v = v.split('=', 1)
                    title = title.strip()
                else:
                    title = k
                v = visitor.parse_text(v.strip(), 'article')
                d[k] = {'title':title, 'content':v}
        if len(block['kwargs']) == 1 and list(block['kwargs'].keys())[0] != 'target':
            key = list(block['kwargs'].keys())[0]
        else:
            key = block['kwargs'].get('target', '')
        if key in data:
            data[key] = data[key].update(d)
        else:
            data[key] = d
        txt.append(json_dumps(data))
        txt.append('</script>')
        return '\n'.join(txt)
    else:
        return code_comment(visitor, block)

def toc(visitor, block, headers=None, relations=None):
    """
    Format:
        
        {% toc max_depth=2 %}
        file1.md
        file2.md
        {% endtoc %}
    """
    s = []
    s.append('<div class="toc">\n')
    depth = int(block['kwargs'].get('max_depth', '1'))
    if 'class' in block['kwargs']:
        s.append('<ul class=%s>\n' % block['kwargs']['class'])
    else:
        s.append('<ul>\n')
    prev = {'prev':{}, 'next':{}, 'current':{}}
    
    for fname in block['body'].splitlines():
        hs = headers.get(fname, [])
        if not hs:
            log.error("File %s can't be found, will be skipped" % fname)
            continue
        
        def make_title(x):
            return '<li><a href="%s">%s</a></li>' % (x['link'], x['title'])
        
        _fname, _ext = os.path.splitext(fname)
        last = 1
        title = _fname
        for x in hs:
            level = x['level']
            
            #fetch title, so an article should be one h1 subject
            if level == 1:
                title = x['title']
                
            if level > depth:
                continue
            if level == last: #same
                s.append(make_title(x))
            elif level == last-1: #reindent
                s.append('</ul>\n')
                s.append(make_title(x))
                last -= 1
            elif level == last+1: #indent
                s.append('<ul>\n')
                s.append(make_title(x))
                last += 1
            else:
                pass
        for i in range(last, 1, -1):
            s.append('</ul>\n')
            
        #process prev and next
        c = {'link':_fname+'.html', 'title':title}
        current = relations[_fname] = {
            'prev':{}, 'next':{}, 
            'current':c,
        }
        p = prev['current']
        if p:
            current['prev'] = {'link':p['link'], 'title':p['title']}
        prev['next'] = c
        prev = current
    s.append('</ul>\n')
    s.append('</div>\n')
    return ''.join(s)

def include(visitor, block):
    """
    include source code from file, the format is:
        
        {% include file=test.js, lines=1-20 30-, language=python, class=linenums %}
        lines_content
        {% endinclude %}
        
        the line 1<= lineno <= 20 and 30 <= lineno will be included
        
        lines_content is line pattern used to match which lines should be included
        the format is:
            
            begin ... end
            
        if you don't want to included `begin` or `end` line, you can add `!` at the 
        end of `begin` or `end`
            
        begin and end are line pattern, and end will not be included
    """
    
    def get_lineno(lines):
        num = []
        for x in lines.split():
            if '-' in x:
                b, e = x.split('-')
                if b.strip():
                    b = int(b)
                else:
                    b = 1
                b = int(b)
                if e.strip():
                    e = int(e)
                else:
                    e = -1
                num.append((b, e))
            else:
                num.append((int(x), int(x)))
        return num
        
    def get_line_index(line, v, index, flag=''):
        """
        flag is 'begin' or 'end'
        """
        if isinstance(v, int):
            return v
        skip = False
        p = v
        if v.endswith('!'):
            skip = True
            p = v[:-1]
            
        if re.search(p, line):
            if skip:
                if flag == 'begin':
                    index = index + 1
                elif flag == 'end':
                    index = index - 1
            return index
        else:
            return v
        
    def test_lineno(n, lineno, index):
        """
        last status
        """
        if index != -1:
            b, e = lineno[index]
            if b<=n and ((e==-1) or n<=e):
                return True, index
        for i, v in enumerate(lineno):
            b, e = lineno[i]
            if b<=n and ((e==-1) or n<=e):
                return True, i
                
        return False, -1
        
    kw = block['kwargs']
    filename = kw.pop('file', '')
    if visitor.filename:
        _dir = os.path.dirname(visitor.filename)
        filename = os.path.join(_dir, filename)
    if not os.path.exists(filename):
        log.error("Can't find the file %s" % filename)
        return '<p class="error">Can\'t find the file %s</p>' % filename
    
    #Get line number array
    lines = kw.pop('lines', '')
    if lines:
        lineno = get_lineno(lines)
    else:
        lineno = []
    
    #process lines_content
    pattern = False
    for x in block['body'].splitlines():
        x = x.rstrip()
        if x:
            v = x.split('...')
            if len(v) == 1:
                b = x
                e = ''
            else:
                b, e = [y.rstrip() for y in v]
            if b.isdigit():
                b = int(b)
            else:
                pattern = True
            b = b or 1
            if e.isdigit():
                e = int(e)
            else:
                pattern = True
            e = e or -1
            if len(v) == 1:
                lineno.append(b)
            else:
                lineno.append((b, e))

    s = []
    first = '```'
    lang = kw.pop('language', '')
    if lang:
        first += lang+','
    x = []
    for k, v in kw.items():
        x.append('%s=%s' % (k, v))
    if x:
        first += ','.join(x)
    s.append(first)
    
    with open(filename, 'r') as f:
        buf = f.readlines()
    
    if lineno and pattern:
        #calculate patterns
        for i, line in enumerate(buf):
            for j, v in enumerate(lineno):
                if isinstance(v, tuple):
                    lineno[j] = (get_line_index(line, v[0], i+1, 'begin'), get_line_index(line, v[1], i+1, 'end'))
                else:
                    t = get_line_index(line, v, i+1)
                    lineno[j] = t, t
        
        for i in range(len(lineno)-1, -1, -1):
            v = lineno[i]
            if not isinstance(v, tuple):
                del lineno[i]
            else:
                if isinstance(v[0], str) or isinstance(v[1], str):
                    del lineno[i]
       
    last_index = -1
    for i, line in enumerate(buf):
        line = line.rstrip()
        if not lineno:
            s.append(line)
        else:
            flag, index = test_lineno(i+1, lineno, last_index)
            if flag:
                s.append(line)
                last_index = index
            else:
                if i-1>=0 and s[-1]!='...':
                    s.append('...')
                last_index = -1
    s.append('```')
    text = '\n'.join(s)
    return visitor.parse_text(text, 'article')
    



__doc__ = """
=====================
Parm 
=====================

:Author: Limodou <limodou@gmail.com>

About Parm
----------------

Parm can be used to convert markdown files to  html pages. It'll use par module 
to parse markdown.
 
The features are:

* Topic content page support
* Bootstrap theme(2.1.1 css framework based)
* Semantic-UI theme

Requirement
----------------

* par https://github.com/limodou/par

Installation
----------------

::
    
    pip install parm

Usage
-------------

::
    
    parm --help
    parm --version
    parm init
    parm make [-d directory]
    parm rst2md

License
------------

Parm is released under BSD license. 

"""

from setuptools import setup
from setuptools.command import build_py as b
import os

#remove build and dist directory
import shutil

def copy_dir(self, package, src, dst):
    self.mkpath(dst)
    for r in os.listdir(src):
        if r in ['.svn', '_svn']:
            continue
        fpath = os.path.join(src, r)
        if os.path.isdir(fpath):
            copy_dir(self, package + '.' + r, fpath, os.path.join(dst, r))
        else:
            ext = os.path.splitext(fpath)[1]
            if ext in ['.pyc', '.pyo', '.bak', '.tmp']:
                continue
            target = os.path.join(dst, r)
            self.copy_file(fpath, target)

def find_dir(self, package, src):
    for r in os.listdir(src):
        if r in ['.svn', '_svn']:
            continue
        fpath = os.path.join(src, r)
        if os.path.isdir(fpath):
            for f in find_dir(self, package + '.' + r, fpath):
                yield f
        else:
            ext = os.path.splitext(fpath)[1]
            if ext in ['.pyc', '.pyo', '.bak', '.tmp']:
                continue
            yield fpath

def build_package_data(self):
    for package in self.packages or ():
        src_dir = self.get_package_dir(package)
        build_dir = os.path.join(*([self.build_lib] + package.split('.')))
        copy_dir(self, package, src_dir, build_dir)
setattr(b.build_py, 'build_package_data', build_package_data)

def get_source_files(self):
    filenames = []
    for package in self.packages or ():
        src_dir = self.get_package_dir(package)
        filenames.extend(list(find_dir(self, package, src_dir)))
    return filenames
setattr(b.build_py, 'get_source_files', get_source_files)

import parm

setup(name='parm',
    version=parm.__version__,
    description="Markdown to html convertor tool",
    long_description=__doc__,
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
    ],
    packages = ['parm'],
    platforms = 'any',
    keywords='markdown convertor',
    author=parm.__author__,
    author_email=parm.__author_email__,
    url=parm.__url__,
    license=parm.__license__,
    include_package_data=True,
    zip_safe=False,
    entry_points = {
        'console_scripts': [
            'parm = parm:main',
        ],
    },
    install_requires=['par']
)
